@@ -30,12 +30,6 @@
 #include <gst/video/video.h>
 #include <gst/gst-i18n-plugin.h>
 
-static const GstElementDetails gst_pngdec_details =
-GST_ELEMENT_DETAILS ("PNG image decoder",
-    "Codec/Decoder/Image",
-    "Decode a png video frame to a raw image",
-    "Wim Taymans <wim@fluendo.com>");
-
 GST_DEBUG_CATEGORY_STATIC (pngdec_debug);
 #define GST_CAT_DEFAULT pngdec_debug
 
@@ -111,16 +105,17 @@ gst_pngdec_base_init (gpointer g_class)
       gst_static_pad_template_get (&gst_pngdec_src_pad_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_pngdec_sink_pad_template));
-  gst_element_class_set_details (element_class, &gst_pngdec_details);
+  gst_element_class_set_details_simple (element_class, "PNG image decoder",
+      "Codec/Decoder/Image",
+      "Decode a png video frame to a raw image",
+      "Wim Taymans <wim@fluendo.com>");
 }
 
 static void
 gst_pngdec_class_init (GstPngDecClass * klass)
 {
-  GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
 
-  gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
@@ -191,7 +186,7 @@ user_info_callback (png_structp png_ptr, png_infop info)
   size_t buffer_size;
   GstBuffer *buffer = NULL;
 
-  pngdec = GST_PNGDEC (png_ptr->io_ptr);
+  pngdec = GST_PNGDEC (png_get_io_ptr (png_ptr));
 
   GST_LOG ("info ready");
 
@@ -203,7 +198,14 @@ user_info_callback (png_structp png_ptr, png_infop info)
 
   /* Allocate output buffer */
   pngdec->rowbytes = png_get_rowbytes (pngdec->png, pngdec->info);
-  buffer_size = pngdec->height * GST_ROUND_UP_4 (pngdec->rowbytes);
+  if (pngdec->rowbytes > (G_MAXUINT32 - 3)
+      || pngdec->height > G_MAXUINT32 / pngdec->rowbytes) {
+    ret = GST_FLOW_ERROR;
+    goto beach;
+  }
+  pngdec->rowbytes = GST_ROUND_UP_4 (pngdec->rowbytes);
+  buffer_size = pngdec->height * pngdec->rowbytes;
+
   ret =
       gst_pad_alloc_buffer_and_set_caps (pngdec->srcpad, GST_BUFFER_OFFSET_NONE,
       buffer_size, GST_PAD_CAPS (pngdec->srcpad), &buffer);
@@ -223,14 +225,14 @@ user_endrow_callback (png_structp png_ptr, png_bytep new_row,
 {
   GstPngDec *pngdec = NULL;
 
-  pngdec = GST_PNGDEC (png_ptr->io_ptr);
+  pngdec = GST_PNGDEC (png_get_io_ptr (png_ptr));
 
   /* FIXME: implement interlaced pictures */
 
   /* If buffer_out doesn't exist, it means buffer_alloc failed, which 
    * will already have set the return code */
   if (GST_IS_BUFFER (pngdec->buffer_out)) {
-    size_t offset = row_num * GST_ROUND_UP_4 (pngdec->rowbytes);
+    size_t offset = row_num * pngdec->rowbytes;
 
     GST_LOG ("got row %u, copying in buffer %p at offset %" G_GSIZE_FORMAT,
         (guint) row_num, pngdec->buffer_out, offset);
@@ -246,15 +248,17 @@ buffer_clip (GstPngDec * dec, GstBuffer * buffer)
   gboolean res = TRUE;
   gint64 cstart, cstop;
 
+
   if ((!GST_CLOCK_TIME_IS_VALID (GST_BUFFER_TIMESTAMP (buffer))) ||
       (!GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DURATION (buffer))) ||
       (dec->segment.format != GST_FORMAT_TIME))
     goto beach;
 
+  cstart = GST_BUFFER_TIMESTAMP (buffer);
+  cstop = GST_BUFFER_DURATION (buffer);
+
   if ((res = gst_segment_clip (&dec->segment, GST_FORMAT_TIME,
-              GST_BUFFER_TIMESTAMP (buffer),
-              GST_BUFFER_TIMESTAMP (buffer) + GST_BUFFER_DURATION (buffer),
-              &cstart, &cstop))) {
+              cstart, cstart + cstop, &cstart, &cstop))) {
     GST_BUFFER_TIMESTAMP (buffer) = cstart;
     GST_BUFFER_DURATION (buffer) = cstop - cstart;
   }
@@ -268,7 +272,7 @@ user_end_callback (png_structp png_ptr, png_infop info)
 {
   GstPngDec *pngdec = NULL;
 
-  pngdec = GST_PNGDEC (png_ptr->io_ptr);
+  pngdec = GST_PNGDEC (png_get_io_ptr (png_ptr));
 
   GST_LOG_OBJECT (pngdec, "and we are done reading this image");
 
@@ -301,17 +305,23 @@ user_read_data (png_structp png_ptr, png_bytep data, png_size_t length)
   GstPngDec *pngdec;
   GstBuffer *buffer;
   GstFlowReturn ret = GST_FLOW_OK;
+  guint size;
 
-  pngdec = GST_PNGDEC (png_ptr->io_ptr);
+  pngdec = GST_PNGDEC (png_get_io_ptr (png_ptr));
 
   GST_LOG ("reading %" G_GSIZE_FORMAT " bytes of data at offset %d", length,
       pngdec->offset);
 
   ret = gst_pad_pull_range (pngdec->sinkpad, pngdec->offset, length, &buffer);
-  if ((ret != GST_FLOW_OK) || (GST_BUFFER_SIZE (buffer) != length))
+  if (ret != GST_FLOW_OK)
     goto pause;
 
-  memcpy (data, GST_BUFFER_DATA (buffer), GST_BUFFER_SIZE (buffer));
+  size = GST_BUFFER_SIZE (buffer);
+
+  if (size != length)
+    goto short_buffer;
+
+  memcpy (data, GST_BUFFER_DATA (buffer), size);
 
   gst_buffer_unref (buffer);
 
@@ -319,14 +329,30 @@ user_read_data (png_structp png_ptr, png_bytep data, png_size_t length)
 
   return;
 
+  /* ERRORS */
 pause:
-  GST_INFO_OBJECT (pngdec, "pausing task, reason %s", gst_flow_get_name (ret));
-  gst_pad_pause_task (pngdec->sinkpad);
-  if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
-    gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
+  {
+    GST_INFO_OBJECT (pngdec, "pausing task, reason %s",
+        gst_flow_get_name (ret));
+    gst_pad_pause_task (pngdec->sinkpad);
+    if (ret == GST_FLOW_UNEXPECTED) {
+      gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
+    } else if (ret < GST_FLOW_UNEXPECTED || ret == GST_FLOW_NOT_LINKED) {
+      GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
+          (_("Internal data stream error.")),
+          ("stream stopped, reason %s", gst_flow_get_name (ret)));
+      gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
+    }
+    return;
+  }
+short_buffer:
+  {
+    gst_buffer_unref (buffer);
     GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
         (_("Internal data stream error.")),
-        ("stream stopped, reason %s", gst_flow_get_name (ret)));
+        ("Read %u, needed %" G_GSIZE_FORMAT "bytes", size, length));
+    ret = GST_FLOW_ERROR;
+    goto pause;
   }
 }
 
@@ -370,7 +396,11 @@ gst_pngdec_caps_create_and_set (GstPngDec * pngdec)
     png_set_gray_to_rgb (pngdec->png);
     if (bpc < 8) {              /* Convert to 8 bits */
       GST_LOG_OBJECT (pngdec, "converting grayscale image to 8 bits");
+#if PNG_LIBPNG_VER < 10400
       png_set_gray_1_2_4_to_8 (pngdec->png);
+#else
+      png_set_expand_gray_1_2_4_to_8 (pngdec->png);
+#endif
     }
   }
 
@@ -476,13 +506,17 @@ gst_pngdec_task (GstPad * pad)
 
   /* Allocate output buffer */
   rowbytes = png_get_rowbytes (pngdec->png, pngdec->info);
-  buffer_size = pngdec->height * GST_ROUND_UP_4 (rowbytes);
+  if (rowbytes > (G_MAXUINT32 - 3) || pngdec->height > G_MAXUINT32 / rowbytes) {
+    ret = GST_FLOW_ERROR;
+    goto pause;
+  }
+  rowbytes = GST_ROUND_UP_4 (rowbytes);
+  buffer_size = pngdec->height * rowbytes;
   ret =
       gst_pad_alloc_buffer_and_set_caps (pngdec->srcpad, GST_BUFFER_OFFSET_NONE,
       buffer_size, GST_PAD_CAPS (pngdec->srcpad), &buffer);
-  if (ret != GST_FLOW_OK) {
+  if (ret != GST_FLOW_OK)
     goto pause;
-  }
 
   rows = (png_bytep *) g_malloc (sizeof (png_bytep) * pngdec->height);
 
@@ -490,18 +524,17 @@ gst_pngdec_task (GstPad * pad)
 
   for (i = 0; i < pngdec->height; i++) {
     rows[i] = inp;
-    inp += GST_ROUND_UP_4 (rowbytes);
+    inp += rowbytes;
   }
 
   /* Read the actual picture */
   png_read_image (pngdec->png, rows);
-  free (rows);
+  g_free (rows);
 
   /* Push the raw RGB frame */
   ret = gst_pad_push (pngdec->srcpad, buffer);
-  if (ret != GST_FLOW_OK) {
+  if (ret != GST_FLOW_OK)
     goto pause;
-  }
 
   /* And we are done */
   gst_pad_pause_task (pngdec->sinkpad);
@@ -509,13 +542,18 @@ gst_pngdec_task (GstPad * pad)
   return;
 
 pause:
-  GST_INFO_OBJECT (pngdec, "pausing task, reason %s", gst_flow_get_name (ret));
-  gst_pad_pause_task (pngdec->sinkpad);
-  if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
-    GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
-        (_("Internal data stream error.")),
-        ("stream stopped, reason %s", gst_flow_get_name (ret)));
-    gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
+  {
+    GST_INFO_OBJECT (pngdec, "pausing task, reason %s",
+        gst_flow_get_name (ret));
+    gst_pad_pause_task (pngdec->sinkpad);
+    if (ret == GST_FLOW_UNEXPECTED) {
+      gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
+    } else if (ret == GST_FLOW_NOT_LINKED || ret < GST_FLOW_UNEXPECTED) {
+      GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
+          (_("Internal data stream error.")),
+          ("stream stopped, reason %s", gst_flow_get_name (ret)));
+      gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
+    }
   }
 }
 
@@ -835,10 +873,6 @@ setup_failed:
 static gboolean
 gst_pngdec_sink_activate_pull (GstPad * sinkpad, gboolean active)
 {
-  GstPngDec *pngdec;
-
-  pngdec = GST_PNGDEC (GST_OBJECT_PARENT (sinkpad));
-
   if (active) {
     return gst_pad_start_task (sinkpad, (GstTaskFunction) gst_pngdec_task,
         sinkpad);
