@@ -43,13 +43,15 @@ G_BEGIN_DECLS
 
 typedef enum {
   GST_MATROSKA_DEMUX_STATE_START,
+  GST_MATROSKA_DEMUX_STATE_SEGMENT,
   GST_MATROSKA_DEMUX_STATE_HEADER,
   GST_MATROSKA_DEMUX_STATE_DATA,
-  GST_MATROSKA_DEMUX_STATE_SEEK
+  GST_MATROSKA_DEMUX_STATE_SEEK,
+  GST_MATROSKA_DEMUX_STATE_SCANNING
 } GstMatroskaDemuxState;
 
 typedef struct _GstMatroskaDemux {
-  GstEbmlRead              parent;
+  GstElement              parent;
 
   /* < private > */
 
@@ -75,6 +77,7 @@ typedef struct _GstMatroskaDemux {
   GstMatroskaDemuxState    state;
   guint                    level_up;
   guint64                  seek_block;
+  gboolean                 seek_first;
 
   /* did we parse cues/tracks/segmentinfo already? */
   gboolean                 index_parsed;
@@ -82,6 +85,7 @@ typedef struct _GstMatroskaDemux {
   gboolean                 segmentinfo_parsed;
   gboolean                 attachments_parsed;
   GList                   *tags_parsed;
+  GList                   *seek_parsed;
 
   /* start-of-segment */
   guint64                  ebml_segment_start;
@@ -101,12 +105,19 @@ typedef struct _GstMatroskaDemux {
   GstEvent                *new_segment;
   GstTagList              *global_tags;
 
-  /* push based mode usual suspects */
+  /* pull mode caching */
+  GstBuffer *cached_buffer;
+
+  /* push and pull mode */
   guint64                  offset;
-  GstAdapter              *adapter;
   /* some state saving */
   GstClockTime             cluster_time;
   guint64                  cluster_offset;
+  guint64                  first_cluster_offset;
+  guint64                  next_cluster_offset;
+
+  /* push based mode usual suspects */
+  GstAdapter              *adapter;
   /* index stuff */
   gboolean                 seekable;
   gboolean                 building_index;
@@ -117,12 +128,10 @@ typedef struct _GstMatroskaDemux {
   /* reverse playback */
   GArray                  *seek_index;
   gint                     seek_entry;
-  gint64                   from_offset;
-  gint64                   to_offset;
 } GstMatroskaDemux;
 
 typedef struct _GstMatroskaDemuxClass {
-  GstEbmlReadClass parent;
+  GstElementClass parent;
 } GstMatroskaDemuxClass;
 
 gboolean gst_matroska_demux_plugin_init (GstPlugin *plugin);
