@@ -25,12 +25,6 @@
 
 G_BEGIN_DECLS
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <errno.h>
-#include <string.h>
-
 #include "gstudpnetutils.h"
 #include "gstudp.h"
 
@@ -44,10 +38,11 @@ typedef struct _GstMultiUDPSink GstMultiUDPSink;
 typedef struct _GstMultiUDPSinkClass GstMultiUDPSinkClass;
 
 typedef struct {
+  gint refcount;
+
   int *sock;
 
-  struct sockaddr_in theiraddr;
-  struct ip_mreq multi_addr;
+  struct sockaddr_storage theiraddr;
 
   gchar *host;
   gint port;
@@ -69,8 +64,22 @@ struct _GstMultiUDPSink {
   GMutex        *client_lock;
   GList         *clients;
 
+  /* properties */
   guint64        bytes_to_serve;
   guint64        bytes_served;
+  int            sockfd;
+  gboolean       closefd;
+
+  gboolean       externalfd;
+
+  gboolean       auto_multicast;
+  gint           ttl;
+  gint           ttl_mc;
+  gboolean       loop;
+  gint           qos_dscp;
+  guint16        ss_family;
+
+  gboolean       send_duplicates;
 };
 
 struct _GstMultiUDPSinkClass {
