@@ -282,7 +282,7 @@ gst_v4l2_probe_get_values (GstPropertyProbe * probe,
 }
 
 #define GST_TYPE_V4L2_DEVICE_FLAGS (gst_v4l2_device_get_type ())
-GType
+static GType
 gst_v4l2_device_get_type (void)
 {
   static GType v4l2_device_type = 0;
@@ -315,23 +315,74 @@ gst_v4l2_object_install_properties_helper (GObjectClass * gobject_class,
 {
   g_object_class_install_property (gobject_class, PROP_DEVICE,
       g_param_spec_string ("device", "Device", "Device location",
-          default_device, G_PARAM_READWRITE));
+          default_device, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_DEVICE_NAME,
       g_param_spec_string ("device-name", "Device name",
-          "Name of the device", DEFAULT_PROP_DEVICE_NAME, G_PARAM_READABLE));
+          "Name of the device", DEFAULT_PROP_DEVICE_NAME,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_DEVICE_FD,
       g_param_spec_int ("device-fd", "File descriptor",
           "File descriptor of the device", -1, G_MAXINT, DEFAULT_PROP_DEVICE_FD,
-          G_PARAM_READABLE));
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_FLAGS,
       g_param_spec_flags ("flags", "Flags", "Device type flags",
-          GST_TYPE_V4L2_DEVICE_FLAGS, DEFAULT_PROP_FLAGS, G_PARAM_READABLE));
+          GST_TYPE_V4L2_DEVICE_FLAGS, DEFAULT_PROP_FLAGS,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstV4l2Src:brightness
+   *
+   * Picture brightness, or more precisely, the black level
+   *
+   * Since: 0.10.26
+   */
+  g_object_class_install_property (gobject_class, PROP_BRIGHTNESS,
+      g_param_spec_int ("brightness", "Brightness",
+          "Picture brightness, or more precisely, the black level", G_MININT,
+          G_MAXINT, 0,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | GST_PARAM_CONTROLLABLE));
+  /**
+   * GstV4l2Src:contrast
+   *
+   * Picture contrast or luma gain
+   *
+   * Since: 0.10.26
+   */
+  g_object_class_install_property (gobject_class, PROP_CONTRAST,
+      g_param_spec_int ("contrast", "Contrast",
+          "Picture contrast or luma gain", G_MININT,
+          G_MAXINT, 0,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | GST_PARAM_CONTROLLABLE));
+  /**
+   * GstV4l2Src:saturation
+   *
+   * Picture color saturation or chroma gain
+   *
+   * Since: 0.10.26
+   */
+  g_object_class_install_property (gobject_class, PROP_SATURATION,
+      g_param_spec_int ("saturation", "Saturation",
+          "Picture color saturation or chroma gain", G_MININT,
+          G_MAXINT, 0,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | GST_PARAM_CONTROLLABLE));
+  /**
+   * GstV4l2Src:hue
+   *
+   * Hue or color balance
+   *
+   * Since: 0.10.26
+   */
+  g_object_class_install_property (gobject_class, PROP_HUE,
+      g_param_spec_int ("hue", "Hue",
+          "Hue or color balance", G_MININT,
+          G_MAXINT, 0,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | GST_PARAM_CONTROLLABLE));
 }
 
 GstV4l2Object *
 gst_v4l2_object_new (GstElement * element,
     enum v4l2_buf_type type,
-    char *default_device,
+    const char *default_device,
     GstV4l2GetInOutFunction get_in_out_func,
     GstV4l2SetInOutFunction set_in_out_func,
     GstV4l2UpdateFpsFunction update_fps_func)
@@ -403,6 +454,30 @@ gst_v4l2_object_clear_format_list (GstV4l2Object * v4l2object)
   return TRUE;
 }
 
+static gint
+gst_v4l2_object_prop_to_cid (guint prop_id)
+{
+  gint cid = -1;
+
+  switch (prop_id) {
+    case PROP_BRIGHTNESS:
+      cid = V4L2_CID_BRIGHTNESS;
+      break;
+    case PROP_CONTRAST:
+      cid = V4L2_CID_CONTRAST;
+      break;
+    case PROP_SATURATION:
+      cid = V4L2_CID_SATURATION;
+      break;
+    case PROP_HUE:
+      cid = V4L2_CID_HUE;
+      break;
+    default:
+      GST_WARNING ("unmapped property id: %d", prop_id);
+  }
+  return cid;
+}
+
 
 gboolean
 gst_v4l2_object_set_property_helper (GstV4l2Object * v4l2object,
@@ -413,6 +488,21 @@ gst_v4l2_object_set_property_helper (GstV4l2Object * v4l2object,
       g_free (v4l2object->videodev);
       v4l2object->videodev = g_value_dup_string (value);
       break;
+    case PROP_BRIGHTNESS:
+    case PROP_CONTRAST:
+    case PROP_SATURATION:
+    case PROP_HUE:
+    {
+      gint cid = gst_v4l2_object_prop_to_cid (prop_id);
+
+      if (cid != -1) {
+        if (GST_V4L2_IS_OPEN (v4l2object)) {
+          gst_v4l2_set_attribute (v4l2object, cid, g_value_get_int (value));
+        }
+      }
+      return TRUE;
+    }
+      break;
 #if 0
     case PROP_NORM:
       if (GST_V4L2_IS_OPEN (v4l2object)) {
@@ -516,6 +606,24 @@ gst_v4l2_object_get_property_helper (GstV4l2Object * v4l2object,
       g_value_set_flags (value, flags);
       break;
     }
+    case PROP_BRIGHTNESS:
+    case PROP_CONTRAST:
+    case PROP_SATURATION:
+    case PROP_HUE:
+    {
+      gint cid = gst_v4l2_object_prop_to_cid (prop_id);
+
+      if (cid != -1) {
+        if (GST_V4L2_IS_OPEN (v4l2object)) {
+          gint v;
+          if (gst_v4l2_get_attribute (v4l2object, cid, &v)) {
+            g_value_set_int (value, v);
+          }
+        }
+      }
+      return TRUE;
+    }
+      break;
     default:
       return FALSE;
       break;
@@ -862,8 +970,8 @@ format_cmp_func (gconstpointer a, gconstpointer b)
   if (fa->pixelformat == fb->pixelformat)
     return 0;
 
-  return gst_v4l2_object_format_get_rank (fa) -
-      gst_v4l2_object_format_get_rank (fb);
+  return gst_v4l2_object_format_get_rank (fb) -
+      gst_v4l2_object_format_get_rank (fa);
 }
 
 /******************************************************
@@ -910,7 +1018,20 @@ gst_v4l2_object_fill_format_list (GstV4l2Object * v4l2object)
         (GCompareFunc) format_cmp_func);
   }
 
-  GST_DEBUG_OBJECT (v4l2object->element, "got %d format(s)", n);
+#ifndef GST_DISABLE_GST_DEBUG
+  {
+    GSList *l;
+
+    GST_INFO_OBJECT (v4l2object->element, "got %d format(s):", n);
+    for (l = v4l2object->formats; l != NULL; l = l->next) {
+      format = l->data;
+
+      GST_INFO_OBJECT (v4l2object->element,
+          "  %" GST_FOURCC_FORMAT "%s", GST_FOURCC_ARGS (format->pixelformat),
+          ((format->flags & V4L2_FMT_FLAG_EMULATED)) ? " (emulated)" : "");
+    }
+  }
+#endif
 
   return TRUE;
 
@@ -1200,7 +1321,6 @@ gst_v4l2_object_get_caps_info (GstV4l2Object * v4l2object, GstCaps * caps,
 
   if (strcmp (mimetype, "video/mpegts") == 0) {
     fourcc = V4L2_PIX_FMT_MPEG;
-    outsize = 8192;
     *fps_n = 0;
     *fps_d = 1;
     goto done;
@@ -1731,7 +1851,7 @@ default_frame_sizes:
     /* Since we can't get framerate directly, try to use the current norm */
     if (v4l2object->norm && v4l2object->norms) {
       GList *norms;
-      GstTunerNorm *norm;
+      GstTunerNorm *norm = NULL;
 
       for (norms = v4l2object->norms; norms != NULL; norms = norms->next) {
         norm = (GstTunerNorm *) norms->data;
@@ -1869,6 +1989,22 @@ gst_v4l2_object_set_format (GstV4l2Object * v4l2object, guint32 pixelformat,
   if (v4l2_ioctl (fd, VIDIOC_G_FMT, &format) < 0)
     goto get_fmt_failed;
 
+  if (format.type == v4l2object->type &&
+      format.fmt.pix.width == width &&
+      format.fmt.pix.height == height &&
+      format.fmt.pix.pixelformat == pixelformat) {
+    /* Nothing to do. We want to succeed immediately
+     * here because setting the same format back
+     * can still fail due to EBUSY. By short-circuiting
+     * here, we allow pausing and re-playing pipelines
+     * with changed caps, as long as the changed caps
+     * do not change the webcam's format. Otherwise,
+     * any caps change would require us to go to NULL
+     * state to close the device and set format.
+     */
+    return TRUE;
+  }
+
   format.type = v4l2object->type;
   format.fmt.pix.width = width;
   format.fmt.pix.height = height;
