@@ -20,23 +20,18 @@
 
 /**
  * SECTION:element-ximagesrc
- * @short_description: a source that captures your X Display
  *
- * <refsect2>
- * <para>
  * This element captures your X Display and creates raw RGB video.  It uses
  * the XDamage extension if available to only capture areas of the screen that
  * have changed since the last frame.  It uses the XFixes extension if
  * available to also capture your mouse pointer.  By default it will fixate to
  * 25 frames per second.
- * </para>
+ *
+ * <refsect2>
  * <title>Example pipelines</title>
- * <para>
- * <programlisting>
- * gst-launch -v ximagesrc ! video/x-raw-rgb,framerate=5/1 ! ffmpegcolorspace ! theoraenc ! oggmux ! filesink location=desktop.ogg
- * </programlisting>
- * Encodes your X display to an Ogg theora video at 5 frames per second.
- * </para>
+ * |[
+ * gst-launch ximagesrc ! video/x-raw-rgb,framerate=5/1 ! ffmpegcolorspace ! theoraenc ! oggmux ! filesink location=desktop.ogg
+ * ]| Encodes your X display to an Ogg theora video at 5 frames per second.
  * </refsect2>
  */
 
@@ -57,15 +52,6 @@
 GST_DEBUG_CATEGORY_STATIC (gst_debug_ximage_src);
 #define GST_CAT_DEFAULT gst_debug_ximage_src
 
-/* elementfactory information */
-static const GstElementDetails ximagesrc_details =
-GST_ELEMENT_DETAILS ("Ximage video source",
-    "Source/Video",
-    "Creates a screenshot video stream",
-    "Lutz Mueller <lutz@users.sourceforge.net>, "
-    "Jan Schmidt <thaytan@mad.scientist.com>, "
-    "Zaheer Merali <zaheerabbas at merali dot org>");
-
 static GstStaticPadTemplate t =
 GST_STATIC_PAD_TEMPLATE ("src", GST_PAD_SRC, GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("video/x-raw-rgb, "
@@ -127,6 +113,13 @@ gst_ximage_src_open_display (GstXImageSrc * s, const gchar * name)
 
   g_mutex_lock (s->x_lock);
   s->xcontext = ximageutil_xcontext_get (GST_ELEMENT (s), name);
+  if (s->xcontext == NULL) {
+    g_mutex_unlock (s->x_lock);
+    GST_ELEMENT_ERROR (s, RESOURCE, OPEN_READ,
+        ("Could not open X display for reading"),
+        ("NULL returned from getting xcontext"));
+    return FALSE;
+  }
   s->width = s->xcontext->width;
   s->height = s->xcontext->height;
 
@@ -154,11 +147,15 @@ gst_ximage_src_open_display (GstXImageSrc * s, const gchar * name)
     int error_base;
     long evmask = NoEventMask;
 
+    s->have_xdamage = FALSE;
+    s->damage = None;
+    s->damage_copy_gc = None;
+    s->damage_region = None;
+
     if (XDamageQueryExtension (s->xcontext->disp, &s->damage_event_base,
             &error_base)) {
       s->damage =
-          XDamageCreate (s->xcontext->disp, s->xwindow,
-          XDamageReportRawRectangles);
+          XDamageCreate (s->xcontext->disp, s->xwindow, XDamageReportNonEmpty);
       if (s->damage != None) {
         s->damage_region = XFixesCreateRegion (s->xcontext->disp, NULL, 0);
         if (s->damage_region != None) {
@@ -198,8 +195,12 @@ gst_ximage_src_start (GstBaseSrc * basesrc)
   GstXImageSrc *s = GST_XIMAGE_SRC (basesrc);
 
   s->last_frame_no = -1;
-
-  return gst_ximage_src_open_display (s, NULL);
+#ifdef HAVE_XDAMAGE
+  if (s->last_ximage)
+    gst_buffer_unref (GST_BUFFER_CAST (s->last_ximage));
+  s->last_ximage = NULL;
+#endif
+  return gst_ximage_src_open_display (s, s->display_name);
 }
 
 static gboolean
@@ -207,10 +208,38 @@ gst_ximage_src_stop (GstBaseSrc * basesrc)
 {
   GstXImageSrc *src = GST_XIMAGE_SRC (basesrc);
 
+#ifdef HAVE_XDAMAGE
+  if (src->last_ximage)
+    gst_buffer_unref (GST_BUFFER_CAST (src->last_ximage));
+  src->last_ximage = NULL;
+#endif
+
   gst_ximage_src_clear_bufpool (src);
 
+#ifdef HAVE_XFIXES
+  if (src->cursor_image)
+    XFree (src->cursor_image);
+  src->cursor_image = NULL;
+#endif
+
   if (src->xcontext) {
     g_mutex_lock (src->x_lock);
+
+#ifdef HAVE_XDAMAGE
+    if (src->damage_copy_gc != None) {
+      XFreeGC (src->xcontext->disp, src->damage_copy_gc);
+      src->damage_copy_gc = None;
+    }
+    if (src->damage_region != None) {
+      XFixesDestroyRegion (src->xcontext->disp, src->damage_region);
+      src->damage_region = None;
+    }
+    if (src->damage != None) {
+      XDamageDestroy (src->xcontext->disp, src->damage);
+      src->damage = None;
+    }
+#endif
+
     ximageutil_xcontext_clear (src->xcontext);
     src->xcontext = NULL;
     g_mutex_unlock (src->x_lock);
@@ -265,14 +294,14 @@ composite_pixel (GstXContext * xcontext, guchar * dest, guchar * src)
       color = *dest;
       break;
     case 16:
-      color = GUINT16_FROM_LE (*(guint32 *) (dest));
+      color = GUINT16_FROM_LE (*(guint16 *) (dest));
       break;
     case 32:
       color = GUINT32_FROM_LE (*(guint32 *) (dest));
       break;
     default:
       /* Should not reach here */
-      g_assert_not_reached ();
+      g_return_if_reached ();
   }
 
   /* possible optimisation:
@@ -332,7 +361,6 @@ static GstXImageSrcBuffer *
 gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
 {
   GstXImageSrcBuffer *ximage = NULL;
-  GstCaps *caps = NULL;
 
   g_mutex_lock (ximagesrc->pool_lock);
   while (ximagesrc->buffer_pool != NULL) {
@@ -350,6 +378,7 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
 
   if (ximage == NULL) {
     GstXContext *xcontext;
+    GstCaps *caps = NULL;
 
     GST_DEBUG_OBJECT (ximagesrc, "creating image (%dx%d)",
         ximagesrc->width, ximagesrc->height);
@@ -360,7 +389,8 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
         (BufferReturnFunc) (gst_ximage_src_return_buf));
     if (ximage == NULL) {
       GST_ELEMENT_ERROR (ximagesrc, RESOURCE, WRITE, (NULL),
-          ("could not create a %dx%d ximage", ximage->width, ximage->height));
+          ("could not create a %dx%d ximage", ximagesrc->width,
+              ximagesrc->height));
       g_mutex_unlock (ximagesrc->x_lock);
       return NULL;
     }
@@ -384,83 +414,110 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
 
     gst_buffer_set_caps (GST_BUFFER (ximage), caps);
     g_mutex_unlock (ximagesrc->x_lock);
+
+    gst_caps_unref (caps);
   }
 
   g_return_val_if_fail (GST_IS_XIMAGE_SRC (ximagesrc), NULL);
 #ifdef HAVE_XDAMAGE
-  if (ximagesrc->have_xdamage && ximagesrc->use_damage) {
+  if (ximagesrc->have_xdamage && ximagesrc->use_damage &&
+      ximagesrc->last_ximage != NULL) {
     XEvent ev;
 
+    /* have_frame is TRUE when either the entire screen has been
+     * grabbed or when the last image has been copied */
+    gboolean have_frame = FALSE;
+
     GST_DEBUG_OBJECT (ximagesrc, "Retrieving screen using XDamage");
 
     do {
       XNextEvent (ximagesrc->xcontext->disp, &ev);
+
       if (ev.type == ximagesrc->damage_event_base + XDamageNotify) {
-        XDamageNotifyEvent *dev = (XDamageNotifyEvent *) & ev;
+        XserverRegion parts;
+        XRectangle *rects;
+        int nrects;
 
-#ifdef HAVE_XSHM
-        if (ximagesrc->xcontext->use_xshm &&
-            dev->area.width == ximagesrc->xcontext->width &&
-            dev->area.height == ximagesrc->xcontext->height) {
-          GST_DEBUG_OBJECT (ximagesrc, "Entire screen was damaged");
-          XShmGetImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
-              ximage->ximage, ximagesrc->startx, ximagesrc->starty, AllPlanes);
-          /* No need to collect more events */
-          while (XPending (ximagesrc->xcontext->disp)) {
-            XNextEvent (ximagesrc->xcontext->disp, &ev);
-          }
-          break;
-        } else
-#endif
-        {
-          /* if we only want a small area, clip this damage region to
-           * area we want */
-          if (ximagesrc->endx > ximagesrc->startx &&
-              ximagesrc->endy > ximagesrc->starty) {
-            /* see if damage area intersects */
-            if (dev->area.x + dev->area.width < ximagesrc->startx ||
-                dev->area.x > ximagesrc->endx) {
-              /* trivial reject */
-            } else if (dev->area.y + dev->area.height < ximagesrc->starty ||
-                dev->area.y > ximagesrc->endy) {
-              /* trivial reject */
-            } else {
-              /* find intersect region */
-              int startx, starty, width, height;
-
-              startx = (dev->area.x < ximagesrc->startx) ? ximagesrc->startx :
-                  dev->area.x;
-              starty = (dev->area.y < ximagesrc->starty) ? ximagesrc->starty :
-                  dev->area.y;
-              width = (dev->area.x + dev->area.width < ximagesrc->endx) ?
-                  dev->area.x + dev->area.width - startx :
-                  ximagesrc->endx - startx;
-              height = (dev->area.y + dev->area.height < ximagesrc->endy) ?
-                  dev->area.y + dev->area.height - starty : ximagesrc->endy -
-                  starty;
-              GST_LOG_OBJECT (ximagesrc,
-                  "Retrieving damaged sub-region @ %d,%d size %dx%d as intersect region",
-                  startx, starty, width, height);
-              XGetSubImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
-                  startx, starty, width, height, AllPlanes, ZPixmap,
-                  ximage->ximage, startx - ximagesrc->startx,
-                  starty - ximagesrc->starty);
-            }
-          } else {
+        parts = XFixesCreateRegion (ximagesrc->xcontext->disp, 0, 0);
+        XDamageSubtract (ximagesrc->xcontext->disp, ximagesrc->damage, None,
+            parts);
+        /* Now copy out all of the damaged rectangles. */
+        rects = XFixesFetchRegion (ximagesrc->xcontext->disp, parts, &nrects);
+        if (rects != NULL) {
+          int i;
 
+          if (!have_frame) {
             GST_LOG_OBJECT (ximagesrc,
-                "Retrieving damaged sub-region @ %d,%d size %dx%d",
-                dev->area.x, dev->area.y, dev->area.width, dev->area.height);
-
-            XGetSubImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
-                dev->area.x, dev->area.y,
-                dev->area.width, dev->area.height,
-                AllPlanes, ZPixmap, ximage->ximage, dev->area.x, dev->area.y);
+                "Copying from last frame ximage->size: %d",
+                GST_BUFFER_SIZE (GST_BUFFER (ximage)));
+            memcpy (GST_BUFFER_DATA (GST_BUFFER (ximage)),
+                GST_BUFFER_DATA (GST_BUFFER (ximagesrc->last_ximage)),
+                GST_BUFFER_SIZE (GST_BUFFER (ximage)));
+            have_frame = TRUE;
           }
+          for (i = 0; i < nrects; i++) {
+            GST_LOG_OBJECT (ximagesrc,
+                "Damaged sub-region @ %d,%d size %dx%d reported",
+                rects[i].x, rects[i].y, rects[i].width, rects[i].height);
+
+            /* if we only want a small area, clip this damage region to
+             * area we want */
+            if (ximagesrc->endx > ximagesrc->startx &&
+                ximagesrc->endy > ximagesrc->starty) {
+              /* see if damage area intersects */
+              if (rects[i].x + rects[i].width < ximagesrc->startx ||
+                  rects[i].x > ximagesrc->endx) {
+                /* trivial reject */
+              } else if (rects[i].y + rects[i].height < ximagesrc->starty ||
+                  rects[i].y > ximagesrc->endy) {
+                /* trivial reject */
+              } else {
+                /* find intersect region */
+                int startx, starty, width, height;
+
+                startx = (rects[i].x < ximagesrc->startx) ? ximagesrc->startx :
+                    rects[i].x;
+                starty = (rects[i].y < ximagesrc->starty) ? ximagesrc->starty :
+                    rects[i].y;
+                width = (rects[i].x + rects[i].width < ximagesrc->endx) ?
+                    rects[i].x + rects[i].width - startx :
+                    ximagesrc->endx - startx;
+                height = (rects[i].y + rects[i].height < ximagesrc->endy) ?
+                    rects[i].y + rects[i].height - starty : ximagesrc->endy -
+                    starty;
+
+                GST_LOG_OBJECT (ximagesrc,
+                    "Retrieving damaged sub-region @ %d,%d size %dx%d as intersect region",
+                    startx, starty, width, height);
+                XGetSubImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
+                    startx, starty, width, height, AllPlanes, ZPixmap,
+                    ximage->ximage, startx - ximagesrc->startx,
+                    starty - ximagesrc->starty);
+              }
+            } else {
+
+              GST_LOG_OBJECT (ximagesrc,
+                  "Retrieving damaged sub-region @ %d,%d size %dx%d",
+                  rects[i].x, rects[i].y, rects[i].width, rects[i].height);
+
+              XGetSubImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
+                  rects[i].x, rects[i].y,
+                  rects[i].width, rects[i].height,
+                  AllPlanes, ZPixmap, ximage->ximage, rects[i].x, rects[i].y);
+            }
+          }
+          free (rects);
         }
       }
     } while (XPending (ximagesrc->xcontext->disp));
-    XDamageSubtract (ximagesrc->xcontext->disp, ximagesrc->damage, None, None);
+    if (!have_frame) {
+      GST_LOG_OBJECT (ximagesrc,
+          "Copying from last frame ximage->size: %d",
+          GST_BUFFER_SIZE (GST_BUFFER (ximage)));
+      memcpy (GST_BUFFER_DATA (GST_BUFFER (ximage)),
+          GST_BUFFER_DATA (GST_BUFFER (ximagesrc->last_ximage)),
+          GST_BUFFER_SIZE (GST_BUFFER (ximage)));
+    }
 #ifdef HAVE_XFIXES
     /* re-get area where last mouse pointer was  but only if in our clipping
      * bounds */
@@ -547,11 +604,12 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
 
     GST_DEBUG_OBJECT (ximagesrc, "Using XFixes to draw cursor");
     /* get cursor */
+    if (ximagesrc->cursor_image)
+      XFree (ximagesrc->cursor_image);
     ximagesrc->cursor_image = XFixesGetCursorImage (ximagesrc->xcontext->disp);
     if (ximagesrc->cursor_image != NULL) {
       int cx, cy, i, j, count;
       int startx, starty, iwidth, iheight;
-      gboolean clipped = FALSE;
       gboolean cursor_in_image = TRUE;
 
       cx = ximagesrc->cursor_image->x - ximagesrc->cursor_image->xhot;
@@ -585,7 +643,6 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
           iheight = (cy + ximagesrc->cursor_image->height < ximagesrc->endy) ?
               cy + ximagesrc->cursor_image->height - starty :
               ximagesrc->endy - starty;
-          clipped = TRUE;
         }
       } else {
         startx = cx;
@@ -599,10 +656,13 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
         for (i = 0; i < count; i++)
           ximagesrc->cursor_image->pixels[i] =
               GUINT_TO_LE (ximagesrc->cursor_image->pixels[i]);
-
         /* copy those pixels across */
-        for (j = starty; j < starty + iheight && j < ximagesrc->height; j++) {
-          for (i = startx; i < startx + iwidth && i < ximagesrc->width; i++) {
+        for (j = starty;
+            j < starty + iheight && j < ximagesrc->starty + ximagesrc->height;
+            j++) {
+          for (i = startx;
+              i < startx + iwidth && i < ximagesrc->startx + ximagesrc->width;
+              i++) {
             guint8 *src, *dest;
 
             src =
@@ -622,7 +682,17 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
     }
   }
 #endif
-
+#ifdef HAVE_XDAMAGE
+  if (ximagesrc->have_xdamage && ximagesrc->use_damage) {
+    /* need to ref ximage to put in last_ximage */
+    gst_buffer_ref (GST_BUFFER (ximage));
+    if (ximagesrc->last_ximage) {
+      gst_buffer_unref (GST_BUFFER (ximagesrc->last_ximage));
+    }
+    ximagesrc->last_ximage = ximage;
+    GST_LOG_OBJECT (ximagesrc, "reffing current buffer for last_ximage");
+  }
+#endif
   return ximage;
 }
 
@@ -649,6 +719,13 @@ gst_ximage_src_create (GstPushSrc * bs, GstBuffer ** buf)
    * before capturing */
 
   GST_OBJECT_LOCK (s);
+  if (GST_ELEMENT_CLOCK (s) == NULL) {
+    GST_OBJECT_UNLOCK (s);
+    GST_ELEMENT_ERROR (s, RESOURCE, FAILED,
+        (_("Cannot operate without a clock")), (NULL));
+    return GST_FLOW_ERROR;
+  }
+
   base_time = GST_ELEMENT_CAST (s)->base_time;
   next_capture_ts = gst_clock_get_time (GST_ELEMENT_CLOCK (s));
   next_capture_ts -= base_time;
@@ -813,7 +890,12 @@ gst_ximage_src_base_init (gpointer g_class)
 {
   GstElementClass *ec = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details (ec, &ximagesrc_details);
+  gst_element_class_set_details_simple (ec, "Ximage video source",
+      "Source/Video",
+      "Creates a screenshot video stream",
+      "Lutz Mueller <lutz@users.sourceforge.net>, "
+      "Jan Schmidt <thaytan@mad.scientist.com>, "
+      "Zaheer Merali <zaheerabbas at merali dot org>");
   gst_element_class_add_pad_template (ec, gst_static_pad_template_get (&t));
 }
 
@@ -822,6 +904,8 @@ gst_ximage_src_dispose (GObject * object)
 {
   /* Drop references in the buffer_pool */
   gst_ximage_src_clear_bufpool (GST_XIMAGE_SRC (object));
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
@@ -843,19 +927,20 @@ gst_ximage_src_get_caps (GstBaseSrc * bs)
 {
   GstXImageSrc *s = GST_XIMAGE_SRC (bs);
   GstXContext *xcontext;
-  gint x, y, width, height;
+  gint width, height;
 
-  if ((!s->xcontext) && (!gst_ximage_src_open_display (s, NULL)))
-    return gst_caps_copy (gst_pad_get_pad_template_caps (GST_BASE_SRC (s)->
-            srcpad));
+  if ((!s->xcontext) && (!gst_ximage_src_open_display (s, s->display_name)))
+    return
+        gst_caps_copy (gst_pad_get_pad_template_caps (GST_BASE_SRC
+            (s)->srcpad));
 
   if (!gst_ximage_src_recalc (s))
-    return gst_caps_copy (gst_pad_get_pad_template_caps (GST_BASE_SRC (s)->
-            srcpad));
+    return
+        gst_caps_copy (gst_pad_get_pad_template_caps (GST_BASE_SRC
+            (s)->srcpad));
 
   xcontext = s->xcontext;
 
-  x = y = 0;
   width = xcontext->width;
   height = xcontext->height;
   if (s->endx > s->startx && s->endy > s->starty) {
@@ -863,8 +948,6 @@ gst_ximage_src_get_caps (GstBaseSrc * bs)
     if (s->startx < xcontext->width && s->endx < xcontext->width &&
         s->starty < xcontext->height && s->endy < xcontext->height) {
       /* values are fine */
-      x = s->startx;
-      y = s->starty;
       s->width = width = s->endx - s->startx;
       s->height = height = s->endy - s->starty;
     } else {
@@ -950,14 +1033,14 @@ gst_ximage_src_class_init (GstXImageSrcClass * klass)
 
   g_object_class_install_property (gc, PROP_DISPLAY_NAME,
       g_param_spec_string ("display-name", "Display", "X Display Name", NULL,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gc, PROP_SCREEN_NUM,
       g_param_spec_uint ("screen-num", "Screen number", "X Screen Number",
-          0, G_MAXINT, 0, G_PARAM_READWRITE));
+          0, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gc, PROP_SHOW_POINTER,
       g_param_spec_boolean ("show-pointer", "Show Mouse Pointer",
           "Show mouse pointer (if XFixes extension enabled)", TRUE,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
    * GstXImageSrc:use-damage
    *
@@ -968,7 +1051,7 @@ gst_ximage_src_class_init (GstXImageSrcClass * klass)
   g_object_class_install_property (gc, PROP_USE_DAMAGE,
       g_param_spec_boolean ("use-damage", "Use XDamage",
           "Use XDamage (if XDamage extension enabled)", TRUE,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
    * GstXImageSrc:startx
    *
@@ -980,7 +1063,7 @@ gst_ximage_src_class_init (GstXImageSrcClass * klass)
   g_object_class_install_property (gc, PROP_STARTX,
       g_param_spec_uint ("startx", "Start X co-ordinate",
           "X coordinate of top left corner of area to be recorded (0 for top left of screen)",
-          0, G_MAXINT, 0, G_PARAM_READWRITE));
+          0, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
    * GstXImageSrc:starty
    *
@@ -1031,6 +1114,7 @@ gst_ximage_src_class_init (GstXImageSrcClass * klass)
 static void
 gst_ximage_src_init (GstXImageSrc * ximagesrc, GstXImageSrcClass * klass)
 {
+  gst_base_src_set_format (GST_BASE_SRC (ximagesrc), GST_FORMAT_TIME);
   gst_base_src_set_live (GST_BASE_SRC (ximagesrc), TRUE);
   gst_pad_set_fixatecaps_function (GST_BASE_SRC_PAD (ximagesrc),
       gst_ximage_src_fixate);
